@@ -80,6 +80,7 @@ namespace Sass {
     extern const char only_kwd[];
     extern const char rgb_kwd[];
     extern const char url_kwd[];
+    // extern const char url_prefix_kwd[];
     extern const char image_url_kwd[];
     extern const char important_kwd[];
     extern const char pseudo_not_kwd[];
@@ -144,6 +145,7 @@ namespace Sass {
 
     // constants for uri parsing (RFC 3986 Appendix A.)
     extern const char uri_chars[];
+    extern const char real_uri_chars[];
 
     // some specific constant character classes
     // they must be static to be useable by lexer
