@@ -6,15 +6,24 @@
 #include "constants.hpp"
 #include "utf8/checked.h"
 
+#include <cmath>
 #include <stdint.h>
 
 namespace Sass {
 
-  #define out_of_memory() do {                    \
-      fprintf(stderr, "Out of memory.\n");    \
-      exit(EXIT_FAILURE);                     \
+  #define out_of_memory() do {            \
+      std::cerr << "Out of memory.\n";    \
+      exit(EXIT_FAILURE);                 \
     } while (0)
 
+  double round(double val)
+  {
+    using namespace std;
+    // sometimes I saw numbers like 25.5 rounded to 25
+    // not sure why converting to a float solves this
+    return ::round(float(val));
+  }
+
   /* Sadly, sass_strdup is not portable. */
   char *sass_strdup(const char *str)
   {
@@ -529,6 +538,15 @@ namespace Sass {
   namespace Util {
     using std::string;
 
+    std::string rtrim(const std::string &str) {
+      std::string trimmed = str;
+      size_t pos_ws = trimmed.find_last_not_of(" \t\n\v\f\r");
+      if (pos_ws != std::string::npos)
+      { trimmed.erase(pos_ws + 1); }
+      else { trimmed.clear(); }
+      return trimmed;
+    }
+
     std::string normalize_underscores(const std::string& str) {
       std::string normalized = str;
       for(size_t i = 0, L = normalized.length(); i < L; ++i) {
